import java.util.*;

class Individual {
	private double[] x;

	public double x(int index) {
		return this.x[index];
	}
	public void set_x(int index, double value) {
		this.x[index] = value;
	}

	protected Individual() {
		this.x = new double[16];
	}
}

class DifferentialEvolution {
	private static LinkedList<Individual> population = new LinkedList<Individual>();
	private static Random rnd = new Random();
	private static final int PopulationSize = 64;
	private static final double F = 1.0;
	private static final double CR = 0.50;
	private static final int N = 16;

	private static Individual bestFitness;

	private double costFunction(Individual in) {
		double cost = 0.0;
		Equation eq = new Equation(in);
		for(int i = 0; i < N; i++) {
			cost += Math.pow(eq.px(i) - in.x(i), 2.0);
			if(in.x(i) < 0.0)
				cost += 100.0*(1.0 - in.x(i));
		}
		return Math.sqrt(cost);
	}
	public static double bestFitness(int index) {
		return bestFitness.x(index);
	}

	protected DifferentialEvolution() {
		rnd.setSeed(System.currentTimeMillis());

		for(int i = 0; i < PopulationSize; i++) {
			Individual individual = new Individual();
			for(int j = 0; j < N; j++)
				individual.set_x(j, 1.0 + 0.10*rnd.nextGaussian());
			population.add(individual);
		}

		for(int i = 0; i < 10000; i++) {
			for(int j = 0; j < PopulationSize; j++) {
				int s = rnd.nextInt(PopulationSize);
				int a, b, c;
				do {
					a = rnd.nextInt(PopulationSize);
				} while(a == s);
				do {
					b = rnd.nextInt(PopulationSize);
				} while(b == a || b == s);
				do {
					c = rnd.nextInt(PopulationSize);
				} while(c == a || c == b || c == s);
				int R = rnd.nextInt(N);

				Individual original = population.get(s);
				Individual candidate = new Individual();
				for(int k = 0; k < N; k++)
					candidate.set_x(k, original.x(k));
				Individual individual1 = population.get(a);
				Individual individual2 = population.get(b);
				Individual individual3 = population.get(c);

				for(int k = 0; k < N; k++) {
					if(rnd.nextInt(N) == R || (10.0 + rnd.nextGaussian())%1 < CR)
						candidate.set_x(k, individual1.x(k) + F*(individual2.x(k) - individual3.x(k)));
				}

				if(costFunction(original) > costFunction(candidate)) {
					population.remove(original);
					population.add(candidate);
				}
			}
		}

		bestFitness = new Individual();
		for(int i = 0; i < N; i++)
			bestFitness.set_x(i, 1.0 + 0.10*rnd.nextGaussian());
		for(int i = 0; i < PopulationSize; i++) {
			Individual individual = population.get(i);
			bestFitness = (costFunction(bestFitness) < costFunction(individual)) ? bestFitness : individual;
		}
	}
}

class Equation {
	private static final double LENGTH_OF_CYCLE = 1.0;
	private static final double LENGTH_OF_PNRD = 0.150;
	private static final int STEPS = 5000;

	private static final double TRANSCRIPTION_RATE = 1800.0;
	private static final double TRANSLATION_RATE = 120.0;
	private static final double HIGH_COPY = 0.10;
	private static final double LOW_COPY = 0.010;
	
	private static final double K_sigmaA_antiA = 1.0;
	private static final double K_sigmaB_antiB = 1.0;
	private static final double K_sigmaC_antiC = 1.0;
	private static final double K_sigmaA_P = 1.0;
	private static final double K_sigmaB_P = 1.0;
	private static final double K_sigmaC_P = 1.0;
	private static final double K_triggerRNA_toehold = 1.0;

	private static final double HL_sigmaA = 0.150;
	private static final double HL_sigmaB = 0.150;
	private static final double HL_sigmaC = 0.150;
	private static final double HL_antiA = 0.150;
	private static final double HL_antiB = 0.150;
	private static final double HL_antiC = 0.150;
	private static final double HL_RNA = 0.050;

	private double sigmaA;
	private double sigmaB;
	private double sigmaC;
	private double antiA;
	private double antiB;
	private double antiC;
	private double mRNA_sigmaA;
	private double mRNA_sigmaB;
	private double mRNA_sigmaC;
	private double mRNA_antiA;
	private double mRNA_antiB;
	private double mRNA_antiC;
	private double cr_mRNA_sigmaA;
	private double cr_mRNA_sigmaB;
	private double cr_mRNA_sigmaC;
	private double triggerRNA;

	private double[] px;

	public double px(int index) {
		return this.px[index];
	}

	protected Equation(Individual individual) {
		this.sigmaA = individual.x(0);
		this.sigmaB = individual.x(1);
		this.sigmaC = individual.x(2);
		this.antiA = individual.x(3);
		this.antiB = individual.x(4);
		this.antiC = individual.x(5);
		this.mRNA_sigmaA = individual.x(6);
		this.mRNA_sigmaB = individual.x(7);
		this.mRNA_sigmaC = individual.x(8);
		this.mRNA_antiA = individual.x(9);
		this.mRNA_antiB = individual.x(10);
		this.mRNA_antiC = individual.x(11);
		this.cr_mRNA_sigmaA = individual.x(12);
		this.cr_mRNA_sigmaB = individual.x(13);
		this.cr_mRNA_sigmaC = individual.x(14);
		this.triggerRNA = individual.x(15);

		double delta = 1.0/STEPS;
		double[] temp = new double[16];
		for(int i = 0; i < (int)(3*LENGTH_OF_CYCLE*STEPS); i++) {
			temp[0] = this.sigmaA + delta*(TRANSLATION_RATE*(this.mRNA_sigmaA + this.cr_mRNA_sigmaA*
				this.triggerRNA/(K_triggerRNA_toehold + this.triggerRNA)) - (Math.log(2.0)/HL_sigmaA)*this.sigmaA);
			temp[1] = this.sigmaB + delta*(TRANSLATION_RATE*(this.mRNA_sigmaB + this.cr_mRNA_sigmaB*
				this.triggerRNA/(K_triggerRNA_toehold + this.triggerRNA)) - (Math.log(2.0)/HL_sigmaB)*this.sigmaB);
			temp[2] = this.sigmaC + delta*(TRANSLATION_RATE*(this.mRNA_sigmaC + this.cr_mRNA_sigmaC*
				this.triggerRNA/(K_triggerRNA_toehold + this.triggerRNA)) - (Math.log(2.0)/HL_sigmaC)*this.sigmaC);
			temp[3] = this.antiA + delta*(TRANSLATION_RATE*this.mRNA_antiA - (Math.log(2.0)/HL_antiA)*this.antiA);
			temp[4] = this.antiB + delta*(TRANSLATION_RATE*this.mRNA_antiB - (Math.log(2.0)/HL_antiB)*this.antiB);
			temp[5] = this.antiC + delta*(TRANSLATION_RATE*this.mRNA_antiC - (Math.log(2.0)/HL_antiC)*this.antiC);
			temp[6] = this.mRNA_sigmaA + delta*(LOW_COPY*TRANSCRIPTION_RATE*this.sigmaA*(1.0 - 
				this.antiA/(K_sigmaA_antiA + this.antiA))/(K_sigmaA_P + this.sigmaA*(1.0 -
				this.antiA/(K_sigmaA_antiA + this.antiA))) - (Math.log(2.0)/HL_RNA)*this.mRNA_sigmaA);
			temp[7] = this.mRNA_sigmaB + delta*(LOW_COPY*TRANSCRIPTION_RATE*this.sigmaB*(1.0 - 
				this.antiB/(K_sigmaB_antiB + this.antiB))/(K_sigmaB_P + this.sigmaB*(1.0 -
				this.antiB/(K_sigmaB_antiB + this.antiB))) - (Math.log(2.0)/HL_RNA)*this.mRNA_sigmaB);
			temp[8] = this.mRNA_sigmaC + delta*(LOW_COPY*TRANSCRIPTION_RATE*this.sigmaC*(1.0 - 
				this.antiC/(K_sigmaC_antiC + this.antiC))/(K_sigmaC_P + this.sigmaC*(1.0 -
				this.antiC/(K_sigmaC_antiC + this.antiC))) - (Math.log(2.0)/HL_RNA)*this.mRNA_sigmaC);
			temp[9] = this.mRNA_antiA + delta*(HIGH_COPY*TRANSCRIPTION_RATE*this.sigmaB*(1.0 -
				this.antiB/(K_sigmaB_antiB + this.antiB))/(K_sigmaB_P + this.sigmaB*(1.0 -
				this.antiB/(K_sigmaB_antiB + this.antiB))) - (Math.log(2.0)/HL_RNA)*this.mRNA_antiA);
			temp[10] = this.mRNA_antiB + delta*(HIGH_COPY*TRANSCRIPTION_RATE*this.sigmaC*(1.0 -
				this.antiC/(K_sigmaC_antiC + this.antiC))/(K_sigmaC_P + this.sigmaC*(1.0 -
				this.antiC/(K_sigmaC_antiC + this.antiC))) - (Math.log(2.0)/HL_RNA)*this.mRNA_antiB);
			temp[11] = this.mRNA_antiC + delta*(HIGH_COPY*TRANSCRIPTION_RATE*this.sigmaA*(1.0 -
				this.antiA/(K_sigmaA_antiA + this.antiA))/(K_sigmaA_P + this.sigmaA*(1.0 -
				this.antiA/(K_sigmaA_antiA + this.antiA))) - (Math.log(2.0)/HL_RNA)*this.mRNA_antiC);
			temp[12] = this.cr_mRNA_sigmaA + delta*(LOW_COPY*TRANSCRIPTION_RATE*this.sigmaC*(1.0 -
				this.antiC/(K_sigmaC_antiC + this.antiC))/(K_sigmaC_P + this.sigmaC*(1.0 -
				this.antiC/(K_sigmaC_antiC + this.antiC))) - (Math.log(2.0)/HL_RNA)*this.cr_mRNA_sigmaA);
			temp[13] = this.cr_mRNA_sigmaB + delta*(LOW_COPY*TRANSCRIPTION_RATE*this.sigmaA*(1.0 -
				this.antiA/(K_sigmaA_antiA + this.antiA))/(K_sigmaA_P + this.sigmaA*(1.0 -
				this.antiA/(K_sigmaA_antiA + this.antiA))) - (Math.log(2.0)/HL_RNA)*this.cr_mRNA_sigmaB);
			temp[14] = this.cr_mRNA_sigmaC + delta*(LOW_COPY*TRANSCRIPTION_RATE*this.sigmaB*(1.0 -
				this.antiB/(K_sigmaB_antiB + this.antiB))/(K_sigmaB_P + this.sigmaB*(1.0 -
				this.antiB/(K_sigmaB_antiB + this.antiB))) - (Math.log(2.0)/HL_RNA)*this.cr_mRNA_sigmaC);
			if(i%(int)(LENGTH_OF_CYCLE*STEPS) < (int)((LENGTH_OF_CYCLE - LENGTH_OF_PNRD)*STEPS))
				temp[15] = this.triggerRNA - delta*(Math.log(2.0)/HL_RNA)*this.triggerRNA;
			else
				temp[15] = this.triggerRNA + delta*(LOW_COPY*TRANSCRIPTION_RATE - (Math.log(2.0)/HL_RNA)*this.triggerRNA);

			this.sigmaA = temp[0];
			this.sigmaB = temp[1];
			this.sigmaC = temp[2];
			this.antiA = temp[3];
			this.antiB = temp[4];
			this.antiC = temp[5];
			this.mRNA_sigmaA = temp[6];
			this.mRNA_sigmaB = temp[7];
			this.mRNA_sigmaC = temp[8];
			this.mRNA_antiA = temp[9];
			this.mRNA_antiB = temp[10];
			this.mRNA_antiC = temp[11];
			this.cr_mRNA_sigmaA = temp[12];
			this.cr_mRNA_sigmaB = temp[13];
			this.cr_mRNA_sigmaC = temp[14];
			this.triggerRNA = temp[15];
		}
		this.px = new double[16];
		for(int i = 0; i < 16; i++)
			this.px[i] = temp[i];
	}
}

class Jacobian {
	private static Individual vector = new Individual();
	private static double[][] jacobian;

	public static Matrix jacobi() {
		Matrix matrix = new Matrix();
		for(int i = 0; i < 16; i++) {
			for(int j = 0; j < 16; j++)
				matrix.set_value(i, j, jacobian[i][j]);
		}
		return matrix;
	}

	protected Jacobian() {
		new DifferentialEvolution();

		for(int i = 0; i < 16; i++)
			vector.set_x(i, DifferentialEvolution.bestFitness(i));
		jacobian = new double[16][];
		for(int i = 0; i < 16; i++)
			jacobian[i] = new double[16];
		double delta = 0.010;
		for(int i = 0; i < 16; i++) {
			Individual left_vector = new Individual();
			Individual right_vector = new Individual();
			for(int j = 0; j < 16; j++) {
				left_vector.set_x(j, vector.x(j));
				right_vector.set_x(j, vector.x(j));
			}
			left_vector.set_x(i, vector.x(i) - delta/2.0);
			right_vector.set_x(i, vector.x(i) + delta/2.0);
			Equation left_eq = new Equation(left_vector);
			Equation right_eq = new Equation(right_vector);
			for(int j = 0; j < 16; j++)
				jacobian[j][i] = ((right_eq.px(j) - right_vector.x(j)) - (left_eq.px(j) - left_vector.x(j)))/delta;
		}
	}
}

class Matrix {
	private double[][] matrix;

	public double value(int i, int j) {
		return this.matrix[i][j];
	}
	public void set_value(int i, int j, double value) {
		this.matrix[i][j] = value;
	}
	public Matrix multiply(Matrix object) {
		Matrix product = new Matrix();
		for(int i = 0; i < 16; i++) {
			for(int j = 0; j < 16; j++) {
				double sum = 0.0;
				for(int k = 0; k < 16; k++)
					sum += this.value(i, k)*object.value(k, j);
				product.set_value(i, j, sum);
			}
		}
		return product;
	}

	protected Matrix() {
		this.matrix = new double[16][];
		for(int i = 0; i < 16; i++)
			this.matrix[i] = new double[16];
	}
}

class LUDecomposition {
	private double[][] a;
	private double[][] l;
	private double[][] u;

	public Matrix l() {
		Matrix matrix = new Matrix();
		for(int i = 0; i < 16; i++) {
			for(int j = 0; j < 16; j++)
				matrix.set_value(i, j, this.l[i][j]);
		}
		return matrix;
	}
	public Matrix u() {
		Matrix matrix = new Matrix();
		for(int i = 0; i < 16; i++) {
			for(int j = 0; j < 16; j++)
				matrix.set_value(i, j, this.u[i][j]);
		}
		return matrix;
	}

	protected LUDecomposition(Matrix matrix) {
		this.a = new double[16][];
		this.l = new double[16][];
		this.u = new double[16][];
		for(int i = 0; i < 16; i++) {
			this.a[i] = new double[16];
			this.l[i] = new double[16];
			this.u[i] = new double[16];
			for(int j = 0; j < 16; j++)
				this.a[i][j] = matrix.value(i, j);
		}
		for(int i = 0; i < 15; i++) {
			this.l[i][i] = this.a[i][i];
			this.u[i][i] = 1.0;
			for(int j = i + 1; j < 16; j++) {
				this.l[j][i] = this.a[j][i];
				this.u[i][j] = this.a[i][j]/this.a[i][i];
			}
			for(int j = i + 1; j < 16; j++) {
				for(int k = i + 1; k < 16; k++)
					this.a[j][k] -= this.l[j][i]*this.u[i][k];
			}
		}
		this.l[15][15] = this.a[15][15];
		this.u[15][15] = 1.0;
	}
}

class Eigenvalue {
	private static Matrix x;

	public static double eigenvalue(int i) {
		return x.value(i, i);
	}

	protected Eigenvalue() {
		new Jacobian();
		x = Jacobian.jacobi();
		for(int i = 0; i < 10000; i++) {
			LUDecomposition lu = new LUDecomposition(x);
			x = lu.u().multiply(lu.l());
		}
	}
}

public class Stability {
	public static void main(String[] args) {
		new Eigenvalue();
		boolean stable = true;
		for(int i = 0; i < 16; i++) {
			if(Eigenvalue.eigenvalue(i) > 0.0)
				stable = false;
		}
		System.out.println(stable ? "Stable Loop." : "Unstable Loop.");
	}
}
